(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.24 2013-12-02 15:28:37 fclement Exp $ *)

module Nat = Unatural;;
module Cmd_opt = Ucommand_option;;

module Opt = Options;;
module Config = Configuration;;

let m_failwith = Upervasives.failwith "Main";;

let main () =
  Cmd_opt.parse_phase Opt.phase_dimension;
  let dim = Opt.get_option_dimension () in
  let cod_impl = Opt.get_option_codomain_implementation () in
  let module Cod =
    (val
      match cod_impl with
      | Opt.Option_Tuple_Cod ->
          if dim = 1 then Codomain.single else
          if dim = 3 then Codomain.triple else
          m_failwith "main"
            (Printf.sprintf
               "Tuple codomain of dimension %d not yet implemented" dim)
      | Opt.Option_Vector_Cod -> Codomain.vector_of_nat (Nat.of_int dim)) in
  let module S = Segmentation.Make (Scalar.Float) (Cod) in
  S.algorithm ();
  if Config.get_report_allocated_memory () then
    begin
      let ppf = Config.get_formatter () in
      let (tam, mu) = Monitor.total_allocated_memory () in
      Format.fprintf ppf "@.Total allocated memory: %g %s@." tam mu;
    end;
  exit 0;
;;

main ();;
