(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ordered_field.ml,v 1.9 2013-12-02 15:28:38 fclement Exp $ *)

module Nat = Unatural;;

open Ordered_field_spec;;

module Make =
  functor (Ofs : ORDERED_FIELD_STRUCTURE) ->
struct
  include Ofs;;
  
  let _2_ = _1_ +. _1_;;
  let _3_ = _2_ +. _1_;;
  let _4_ = _3_ +. _1_;;
  let _5_ = _4_ +. _1_;;
  let _6_ = _5_ +. _1_;;
  let _7_ = _6_ +. _1_;;
  let _8_ = _7_ +. _1_;;
  let _9_ = _8_ +. _1_;;
  
  let minus_1_ = ~-. _1_
  and minus_2_ = ~-. _2_
  and minus_3_ = ~-. _3_
  and minus_4_ = ~-. _4_
  and minus_5_ = ~-. _5_
  and minus_6_ = ~-. _6_
  and minus_7_ = ~-. _7_
  and minus_8_ = ~-. _8_
  and minus_9_ = ~-. _9_
  ;;
  
  let inv_2_ = ~/. _2_
  and inv_3_ = ~/. _3_
  and inv_4_ = ~/. _4_
  and inv_5_ = ~/. _5_
  and inv_6_ = ~/. _6_
  and inv_7_ = ~/. _7_
  and inv_8_ = ~/. _8_
  and inv_9_ = ~/. _9_
  ;;
  
  let ( -. ) x y = x +. (~-. y);;
  let ( /. ) x y = x *. (~/. y);;
  let axpy a x y = a *. x +. y;;
  let mean x y = (x +. y) /. _2_;;
  let square x = x *. x;;
  
  let of_nat n = of_int (Nat.to_int n);;
  let to_nat x = Nat.of_int (to_int x);;
end
;;


module Float_structure =
struct
  type t = float;;
  
  (*let digest = Utype_digest.float_digest;;*)
  
  let _0_ = 0.
  and _1_ = 1.
  ;;
  
  let abs = abs_float;;
  
  let is_zero coeff x = abs x <= coeff *. epsilon_float;;
  
  let ( +. ) = Pervasives.( +. );;
  let ( ~-. ) = Pervasives.( ~-. );;
  let ( *. ) = Pervasives.( *. );;
  let ( ~/. ) x = Pervasives.( /. ) _1_ x;;
  
  let of_int = Pervasives.float_of_int;;
  let to_int = Upervasives.round;;
  
  let of_bit_string = Upervasives.float_of_bit_string;;
  let to_bit_string = Upervasives.bit_string_of_float;;
  
  let print = Lib_print.print_quoted_float;;
  let printf oc = Printf.fprintf oc "%F";;
  
  let compare = Pervasives.compare;;
end
;;

module Float = Make (Float_structure);;
