(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: tuple_spec.mli,v 1.25 2013-09-19 10:42:00 fclement Exp $ *)

(** Homogeneous tuples specification. *)

type component_number = Unatural.t;;
  
module type TUPLE_STRUCTURE =
sig
  type 'a t;;
  
  (*val digest : ('a, 'a t) Utype_digest.lifter;;*)
  
  val print : ('a, 'a t) Format_printer.printer_lifter;;
  
  val get : 'a t -> component_number -> 'a;;
  val init : (component_number -> 'a) -> 'a t;;
  val of_list : 'a list -> 'a t;;
  val to_list : 'a t -> 'a list;;
  val of_triple : 'a * 'a * 'a -> 'a t;;
  val to_triple : 'a t -> 'a * 'a * 'a;;
  val iter : ('a -> unit) -> 'a t -> unit;;
  val iter_rails : ('a -> unit) t -> 'a t -> unit;;
  val fold_left : ('accu -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu;;
  val fold_left_i :
    ('accu -> component_number -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu
  ;;
  val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit;;
  val fold_left2 :
    ('accu -> 'a -> 'b -> 'accu) -> 'accu -> 'a t -> 'b t -> 'accu
  ;;
  val iter3 : ('a -> 'b -> 'c -> unit) -> 'a t -> 'b t -> 'c t -> unit;;
  val iter4 :
    ('a -> 'b -> 'c -> 'd -> unit) -> 'a t -> 'b t -> 'c t -> 'd t -> unit
  ;;
end
;;

module type TUPLE =
sig
  include TUPLE_STRUCTURE;;
  
  val make : 'a -> 'a t;;
  
  val same : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool;;
  
  val singleton : component_number -> 'a -> 'a -> 'a t;;
  val map : ('a -> 'b) -> 'a t -> 'b t;;
  val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t;;
  val map3 : ('a -> 'b -> 'c -> 'd) -> 'a t -> 'b t -> 'c t -> 'd t;;
  val map4 :
    ('a -> 'b -> 'c -> 'd -> 'e) -> 'a t -> 'b t -> 'c t -> 'd t -> 'e t
  ;;
  val map_rails : ('a -> 'b) t -> 'a t -> 'b t;;
  val map_rails2 : ('a -> 'b -> 'c) t -> 'a t -> 'b t -> 'c t;;
  val iter_rails3 :
    ('a -> 'b -> 'c -> unit) t -> 'a t -> 'b t -> 'c t -> unit
  ;;
  
  val components : ('a -> 'b t) -> ('a -> 'b) t;;
  val array_components : 'a t array -> 'a array t;;
  val components_map : ('a -> 'b) -> ('c -> 'a) t -> ('c -> 'b) t;;
  
  val split : ('a * 'b) t -> 'a t * 'b t;;
  
  val combine :  ('a -> 'b) t -> 'a -> 'b t;;
end
;;

module type TUPLE_MAKER =
  functor (Ts : TUPLE_STRUCTURE) ->
  TUPLE
;;
