(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of ocamlprintc.bin generated code *)

let print_purpose ppf = function
  | Udebug.User -> Format.fprintf ppf "%s" "User"
  | Udebug.Library -> Format.fprintf ppf "%s" "Library"

and print_software_name ppf = function
  | s -> Lib_print.print_quoted_string ppf s

and print_prefix ppf = function | s -> Lib_print.print_quoted_string ppf s
;;

let rec print_short_key_prefix ppf = function
  | Udebug.Automatic -> Format.fprintf ppf "%s" "Automatic"
  | Udebug.Short p ->
    Format.fprintf ppf "@[<1>%s@ " "Short"; print_prefix ppf p;
    Format.fprintf ppf "%,@]"

and print_key_prefixes ppf = function
  | { Udebug.long = p; Udebug.short = s; } ->
    Format.fprintf ppf "%,@[<1>{@,"; Format.fprintf ppf "%,@[<1>long =@ ";
    print_prefix ppf p; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>short =@ "; print_short_key_prefix ppf s;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_printing_facilities pp_a ppf = function
  | {
      Udebug.report = f_u_m_b;
      Udebug.report_action = f_f_u_u_m_b;
      Udebug.debug = f_u_f_f_u_m_b;
      Udebug.debug_action = f_u_f_f_f_u_u_m_b;
      Udebug.fatal_error = f_u_f_f_u_m_f_i_a;
    } ->
    Format.fprintf ppf "%,@[<1>{@,"; Format.fprintf ppf "%,@[<1>report =@ ";
    Lib_print.print_fun Upervasives_print.print_message Lib_print.print_bool
     ppf f_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>report_action =@ ";
    Lib_print.print_fun
     (Lib_print.print_fun Lib_print.print_unit
       Upervasives_print.print_message
     ) Lib_print.print_bool ppf f_f_u_u_m_b;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@[<1>debug =@ ";
    Lib_print.print_fun Upervasives_print.print_function_name
     (Lib_print.print_fun Upervasives_print.print_message
       Lib_print.print_bool
     ) ppf f_u_f_f_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>debug_action =@ ";
    Lib_print.print_fun Upervasives_print.print_function_name
     (Lib_print.print_fun
       (Lib_print.print_fun Lib_print.print_unit
         Upervasives_print.print_message
       ) Lib_print.print_bool
     ) ppf f_u_f_f_f_u_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>fatal_error =@ ";
    Lib_print.print_fun Upervasives_print.print_function_name
     (Lib_print.print_fun Upervasives_print.print_message
       (Lib_print.print_fun Lib_print.print_quoted_int pp_a)
     ) ppf f_u_f_f_u_m_f_i_a;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let rec print_tag ppf = function | s -> Lib_print.print_quoted_string ppf s

and print_dynamic_tag pp_a ppf = function
  | f_a_t -> Lib_print.print_fun pp_a print_tag ppf f_a_t
;;

let print_dynamic_printing_facilities pp_a pp_b ppf = function
  | {
      Udebug.dyn_report = f_a_f_u_m_b;
      Udebug.dyn_report_action = f_a_f_f_u_u_m_b;
      Udebug.dyn_debug = f_a_f_u_f_f_u_m_b;
      Udebug.dyn_debug_action = f_a_f_u_f_f_f_u_u_m_b;
      Udebug.dyn_fatal_error = f_a_f_u_f_f_u_m_f_i_b;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>dyn_report =@ ";
    Lib_print.print_fun pp_a
     (Lib_print.print_fun Upervasives_print.print_message
       Lib_print.print_bool
     ) ppf f_a_f_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>dyn_report_action =@ ";
    Lib_print.print_fun pp_a
     (Lib_print.print_fun
       (Lib_print.print_fun Lib_print.print_unit
         Upervasives_print.print_message
       ) Lib_print.print_bool
     ) ppf f_a_f_f_u_u_m_b;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@[<1>dyn_debug =@ ";
    Lib_print.print_fun pp_a
     (Lib_print.print_fun Upervasives_print.print_function_name
       (Lib_print.print_fun Upervasives_print.print_message
         Lib_print.print_bool
       )
     ) ppf f_a_f_u_f_f_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>dyn_debug_action =@ ";
    Lib_print.print_fun pp_a
     (Lib_print.print_fun Upervasives_print.print_function_name
       (Lib_print.print_fun
         (Lib_print.print_fun Lib_print.print_unit
           Upervasives_print.print_message
         ) Lib_print.print_bool
       )
     ) ppf f_a_f_u_f_f_f_u_u_m_b;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>dyn_fatal_error =@ ";
    Lib_print.print_fun pp_a
     (Lib_print.print_fun Upervasives_print.print_function_name
       (Lib_print.print_fun Upervasives_print.print_message
         (Lib_print.print_fun Lib_print.print_quoted_int pp_b)
       )
     ) ppf f_a_f_u_f_f_u_m_f_i_b;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

(* End of ocamlprintc.bin generated code *)
