(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ustructural_mapping.mli,v 1.1 2013/06/26 17:29:48 fclement Exp $ *)

(** {3 Structural mappings} *)

type block_number = Udelayed_graph.arrow_number;;

type ('a, 'b, 'c) block = (('a, 'b) Ugraph.t, 'c) Udelayed_graph.arrow;;

type ('a, 'b, 'c) t = (('a, 'b) Ugraph.t, 'c) Udelayed_graph.t;;
(** Blocks are meant to be supported by disjoint sub-domains, and the
 disjoint union of all sub-domains is meant to be the full universe. *)

val get : ('a, 'b, 'c) t -> block_number -> ('a, 'b, 'c) block;;
val set_next : ('a, 'b, 'c) t -> ('a, 'b) Ugraph.t -> ('a, 'b, 'c) t;;
val set :
  ('a, 'b, 'c) t -> block_number -> ('a, 'b) Ugraph.t -> ('a, 'b, 'c) t
;;

val same : ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool;;

(** {6 Converters} *)

val to_graph : ('a, 'b, 'c) t -> ('a, 'b) Ugraph.t;;
val of_graph :
  (('a, 'b) Ugraph.t -> 'c) -> ('a, 'b) Ugraph.t -> ('a, 'b, 'c) t
;;

(** {6 Iterators} *)

val fold_left_i :
  ('accu -> block_number -> ('a, 'b, 'c) block -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;
